#!/usr/bin/env python
import sys
import xml.etree.cElementTree as ET


class Galaxy(object):

    def __init__(self, argv=['script.py']):
        self.argv = argv
        self.executable = self.argv[0]

    def gen(self, param_collection=None, options=[], defaults=[], outputs=[],
            appinfo={}, tests=[], docstring=None, argv=[]):
        #opt_spec = pC.params()

        tool = ET.Element("tool")
        tool.set("id", appinfo['id'])
        tool.set("name", appinfo['name'])
        tool.set("version", appinfo['vers'])

        # Add all of our sections, passing a single xml_writer around.
        self.description_section(tool, appinfo['desc'])
        self.version_section(tool)
        self.stdio_section(tool)

        if docstring is not None:
            self.docstring = docstring
        else:
            self.docstring = "TODO: Write docstring for " + self.executable

        opt_spec = param_collection.params
        self.command_section(tool, opt_spec)
        self.input_section(tool, opt_spec)
        self.output_section(tool, opt_spec)
        self.help_section(tool, self.docstring)
        self.test_section(tool, tests)
        return ET.tostring(tool)

    def description_section(self, parent, appdesc):
        desc = ET.SubElement(parent, 'description')
        desc.text = appdesc

    def version_section(self, parent):
        vers = ET.SubElement(parent, 'version_command')
        vers.text = "python %s --version" % (self.executable, )

    def stdio_section(self, parent):
        stdio = ET.SubElement(parent, 'stdio')
        exit_code = ET.SubElement(stdio, 'exit_code')
        exit_code.set('range', '1:')
        exit_code.set('level', 'fatal')

    def command_section(self, parent, options):
        cmd = ET.SubElement(parent, 'command')
        cmd.set("interpreter", "python")

        command_string = '\n'.join([self.executable, '--galaxy',
                                   '--outfile_supporting $__new_file_path__']) + "\n"

        for command in options:
            addition = command.galaxy_command()
            if (not command._galaxy_specific and command._show_in_galaxy) or \
                (command._galaxy_specific and command._show_in_galaxy):
                if addition:
                    command_string += addition + "\n"

        cmd.text = command_string

    def input_section(self, parent, options):
        inputs = ET.SubElement(parent, 'inputs')
        for option in options:
            if not option.hidden:
                option.galaxy_input(inputs)

    def output_section(self, parent, options):
        outputs = ET.SubElement(parent, 'outputs')
        for option in options:
            option.galaxy_output(outputs)

    def help_section(self, parent, docstring):
        help_sec = ET.SubElement(parent, 'help')
        help_sec.text = docstring.lstrip()

    def test_section(self, parent, test_cases):
        tests = ET.SubElement(parent, 'tests')
        for test in test_cases:
            test_elem = ET.SubElement(tests, 'test')

            # Set parameters
            for param in test['params']:
                param_elem = ET.SubElement(test_elem, 'param')
                param_elem.set('name', param)
                param_elem.set('value', test['params'][param])

            # Files
            for output in test['outputs']:
                output_cmp = test['outputs'][output]
                output_elem = ET.SubElement(test_elem, 'output')
                output_elem.set('name', output)
                output_elem.set('file', output_cmp[1])




#!/usr/bin/env python
from parameter.empty import Empty
from parameter.flag import Flag
from parameter.float import Float
from parameter.genomictagoption import GenomicTagOption
from parameter.int import Int
from parameter.label import Label
from parameter.option import Option
from parameter.fileinput import FileInput
from parameter.fileoutput import FileOutput
from parameter.fileoutputformat import FileOutputFormat
from parameter.string import String
from exceptions import AuthorParameterSpecificationError, ParameterValidationError

import argparse
import warnings


class DefaultList(list):
    def __copy__(self):
        return []


class ParameterCollection(object):

    def __init__(self, desc, vers='1.0.0', argv=[]):
        self.params = []
        self.argv = argv
        self.cur_group = None
        # Add in all arguments as understood
        self.argparser = argparse.ArgumentParser(description=desc)
        self.argparser.add_argument('--version', action='version',
                                    version='%(prog)s ' + str(vers))
        self.argparser.add_argument('-v', '--verbose', action='count', help='Produce more verbose output')

    def validate(self):
        issue_count = 0
        for item in self.params:
            # TODO, check if argparse has a value for the params
            if not item.validate(self.parsed_args[item.name]):
                # print to stderr the errors
                issue_count += 1
        return issue_count == 0

    def push_param(self, param):
        self._push(param)

    def push_params(self, params):
        for p in params:
            self.push_param(p)

    # _add_param
    def _push(self, param):
        p = self._coerce_param(param)
        if p is not None:
            param_extracted = self.convert_params(p)
            argument_flags = param_extracted[0]
            extended_options = param_extracted[1]
            if self.get_current_group() is not None:
                self.get_current_group().add_argument(*argument_flags,
                                                      **extended_options)
            else:
                self.argparser.add_argument(*argument_flags,
                                            **extended_options)
            # Store a copy of the param
            self.params.append(p)

    def convert_params(self, parameter_subclass_object):
        positional_args = []
        if parameter_subclass_object.short is not None:
            positional_args.append('-' +
                                   parameter_subclass_object.short)
        if parameter_subclass_object.name is not None:
            positional_args.append('--' +
                                   parameter_subclass_object.name)

        extended_args = self.sanitise_attrs(parameter_subclass_object)

        return [positional_args, extended_args]

    def sanitise_attrs(self, parameter_subclass_object):
        clean_attr = {}
        attr = parameter_subclass_object.get_all_keys()
        for key in attr:
            if key == 'action':
                if attr[key] in ('store', 'store_const', 'count', 'version',
                                 'help'):
                    clean_attr[key] = attr[key]
            elif key in ('const', 'default', 'version'):
                clean_attr[key] = attr[key]

        # Multiple values
        if parameter_subclass_object.multiple:
            clean_attr['action'] = 'append'
        else:
            clean_attr['action'] = 'store'

        # Handle object type
        obj_type = type(parameter_subclass_object).__name__
        if obj_type == 'Option':
            try:
                clean_attr['choices'] = attr['options']
            except:
                raise AuthorParameterSpecificationError("Author failed to specify 'options' for an Option type parameter")
        elif obj_type == 'GenomicTagOption':
            clean_attr['choices'] = attr['options']
        elif obj_type == 'Int':
            clean_attr['type'] = int
        elif obj_type == 'Float':
            clean_attr['type'] = float
        elif obj_type == 'File':
            clean_attr['type'] = file
        elif obj_type == 'FileInput':
            clean_attr['type'] = argparse.FileType('r')
        elif obj_type == 'FileOutput':
            clean_attr['type'] = str
        elif obj_type == 'Flag':
            clean_attr['action'] = 'store_const'
            clean_attr['const'] = True
        elif obj_type == 'String':
            clean_attr['type'] = str
        elif obj_type == 'FileOutputFormat':
            clean_attr['type'] = str
        else:
            warnings.warn("I don't know how to handle a type of '%s'" %
                          (type(parameter_subclass_object), ), UserWarning)

        # Copy desc
        if parameter_subclass_object.description is not None:
            help_text = parameter_subclass_object.description

        # If there's a defafult, copy over and then add to description
        if parameter_subclass_object.default is not None:
            if parameter_subclass_object.multiple and not isinstance(parameter_subclass_object.default, list):
                raise AuthorParameterSpecificationError("Author declared a multiple attribute, then provided a default value as a scalar, not a list")

            # http://bugs.python.org/issue16399
            # http://www.gossamer-threads.com/lists/python/python/862097
            if parameter_subclass_object.multiple:
                clean_attr['default'] = \
                    DefaultList(parameter_subclass_object.default)
            else:
                clean_attr['default'] = parameter_subclass_object.default
            help_text += " (Default: %(default)s)"

        if isinstance(parameter_subclass_object, Option):
            help_text += " (Options: "
            pso_options = parameter_subclass_object.child_params['options']
            help_text += ', '.join(map(lambda key: "%s [%s]" %
                                       (pso_options[key], key), pso_options))

        if parameter_subclass_object.required:
            help_text += " [Required]"

        clean_attr['help'] = help_text

        if parameter_subclass_object.hidden:
            clean_attr['help'] = argparse.SUPPRESS

        if 'action' not in clean_attr.keys() \
            or ('action' in clean_attr.keys() and clean_attr['action'] !=
                'version' and clean_attr['action'] != 'count' and
                clean_attr['action'] != 'store_const'):
            clean_attr['nargs'] = '?'

        return clean_attr

    def parse_short_name(self, name):
        try:
            return name[name.index('|') + 1:]
        except:
            return None

    def parse_long_name(self, name):
        try:
            return name[0:name.index('|')]
        except:
            return name

    def _coerce0(self, parts=[]):
        p = Empty()
        return p

    def _coerce1(self, parts=[]):
        p = Label(label=parts[0])
        self.add_group(parts[0])
        return p

    def _coerce2(self, parts=[]):
        p = Flag(name=self.parse_long_name(parts[0]),
                 short=self.parse_short_name(parts[0]), multiple=False,
                 description=parts[1],)
        return p

    def _coerce3(self, parts=[]):
        attr = {'name': self.parse_long_name(parts[0]), 'short':
                self.parse_short_name(parts[0]), 'multiple': False,
                'description': parts[1]}

        set_attr = parts[2]

        # Some keys we copy straight over
        for opt in "default options required hidden implies multiple _show_in_galaxy _galaxy_specific _cheetah_var data_format default_format file_format".split(' '):
            if opt in set_attr:
                attr[opt] = set_attr[opt]

        if 'validate' in set_attr:
            validate = set_attr['validate']
            if validate == 'Flag':
                return Flag(**attr)
            elif validate == 'Float':
                for opt in ['min', 'max']:
                    if opt in set_attr:
                        attr[opt] = set_attr[opt]
                return Float(**attr)
            elif validate == 'Int':
                for opt in ['min', 'max']:
                    if opt in set_attr:
                        attr[opt] = set_attr[opt]
                return Int(**attr)
            elif validate == 'Option':
                for opt in ['options']:
                    if opt in set_attr:
                        attr[opt] = set_attr[opt]
                return Option(**attr)
            elif validate == 'String':
                return String(**attr)
            elif validate == 'File/Input':
                return FileInput(**attr)
            elif validate == 'File/Output':
                return FileOutput(**attr)
            elif validate == 'Genomic/Tag':
                return GenomicTagOption(**attr)
            elif validate == 'File/OutputFormat':
                return FileOutputFormat(**attr)
            else:
                raise ParameterValidationError("Error validating parameter type: " + validate)
        else:
            return Flag(**attr)

    def _coerce_param(self, param=[]):
        # These are not "real" parameters
        if len(param) == 0:
            # While technically "good", this code is compeltely
            # useless/nonfunctional and can be removed without issue
            # self._coerce0(param)
            return None
        elif len(param) == 1:
            self._coerce1(param)
            return None
        # Real parameters have <st>curves</st>attributes and help text
        elif len(param) == 2:
            return self._coerce2(param)
        elif len(param) == 3:
            return self._coerce3(param)

    def get_by_name(self, name):
        for item in self.params:
            if item.name == name:
                return item
        return None

    def getarg(self, argv_override=None):
        # Run with data
        if argv_override is not None:
            args = self.argparser.parse_args(argv_override)
        else:
            args = self.argparser.parse_args(self.argv[1:])

        self.parsed_args = vars(args)
        return self.parsed_args

    def get_current_group(self):
        return self.cur_group

    def add_group(self, name):
        self.cur_group = self.argparser.add_argument_group(name)



from parameter import Parameter


class Empty(Parameter):

    def galaxy_input(self, xml_node):
        return None

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, value):
        return True



from ..exceptions import WriterProcessingIncompleteError


class Writer(object):
    suffix = 'ext'

    def __init__(self):
        self.OutputFilesClass = None
        self.galaxy_override = False
        self.data = None
        self.processed_data = None
        self.processing_complete = False
        self.used_filenames = []
        self.name = None

    def process(self):
        self.processed_data = self.data
        self.processing_complete = True

    def write(self):
        if self.processing_complete:
            # Set the extension in the class that generates the filename
            # TBH this can probably be changed from a method to a var
            self.OutputFilesClass.extension = self.suffix
            # Get next filename
            next_output_file = self.OutputFilesClass.get_next_file()
            # Store the filename
            self.used_filenames.append(next_output_file)
            # Write the data
            with open(next_output_file, 'w') as outfile:
                outfile.write(self.processed_data)
        else:
            raise WriterProcessingIncompleteError("""Write called but processing was not marked as
                            complete. Not writing""")

    def get_name(self):
        return self.OutputFilesClass._get_filename()



from parameter import Parameter


class FileOutputFormat(Parameter):

    def galaxy_input(self, xml_node):
        return None

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, val):
        return True



from parameter import Parameter
import xml.etree.cElementTree as ET


class Int(Parameter):

    # Min, Max
    def galaxy_input(self, xml_node):
        node_of_interest = None
        # Due to differences between python/perl, what we do here is figure out
        # which node we actually want to be talking to (main node or the repeat
        # node within it). We can then work with a generic xml_node
        possible_repeat_node = \
            self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('integer')
        if 'min' in self.child_params and self.child_params['min'] is not None:
            params['min'] = str(self.child_params['min'])
        if 'max' in self.child_params and self.child_params['max'] is not None:
            params['max'] = str(self.child_params['max'])

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, val):
        try:
            int(val)

            if 'max' in self.child_params and self.child_params['max'] \
                    is not None:
                if val > self.child_params['max']:
                    self.errors.append('Value passed with %s was greater' +
                                       'than the allowable upper bound. ' +
                                       '[%s > %s]'
                                       % (self.name, val,
                                          self.child_params['max']))
                    return False
            if 'min' in self.child_params and self.child_params['min'] \
                    is not None:
                if val < self.child_params['min']:
                    self.errors.append('Value passed with %s was less' +
                                       'than the allowable minimum bound.'
                                       + '[%s > %s]'
                                       % (self.name, val,
                                          self.child_params['min']))
                    return False
        except ValueError:
            self.errors.append("Not an int")
            return False
        return True



class ParameterValidationError(Exception):
    pass

class UnknownRequestedOutputFileError(Exception):
    pass

class UnspecifiedGGOObjectError(Exception):
    pass

class UnspecifiedOutputFileError(Exception):
    pass

class UnacceptableOutputFormatError(Exception):
    pass

class NoAvailableOutputHandlerError(Exception):
    pass

class UnknownStrategyError(Exception):
    pass

class AuthorParameterSpecificationError(Exception):
    pass

class WriterProcessingIncompleteError(Exception):
    pass

class UnimplementedException(Exception):
    """Maybe this should email me?"""
    pass

class RepeatRequestedFromNonMultiple(Exception):
    pass

class UnknownDataHandlerException(Exception):
    pass

class UnknownDataFormatException(Exception):
    pass



from jinja2 import Template
import sys


class TestGenerator(object):

    def gen(self, tests=[]):
        template = Template("""#!/usr/bin/env python
import difflib
import unittest
import shlex, subprocess
import os

class TestScript(unittest.TestCase):

    def setUp(self):
        self.base = ['python', '{{ script_name }}']
        self.tests = {{ tests }}
    def test_run(self):
        for test_case in self.tests:
            failed_test = False
            try:
                current_command = self.base + \\
                    shlex.split(test_case['command_line'])
                # Should probably be wrapped in an assert as well
                subprocess.check_call(current_command)

                for fileset in test_case['outputs']:
                    failed_test = self.file_comparison(
                            test_case['outputs'][fileset][0],
                            test_case['outputs'][fileset][1])
            except:
                raise
            self.assertFalse(failed_test)

    def file_comparison(self, test_file, comp_file):
        failed_test = False
        diff=difflib.unified_diff(open(test_file).readlines(),
                           open(comp_file).readlines())
        try:
            while True:
                print diff.next(),
                failed_test = True
        except:
            pass
        try:
            # Attempt to remove the generated file to cut down on
            # clutter/other bad things
            os.unlink(test_file)
        except:
            pass
        return failed_test

if __name__ == '__main__':
    unittest.main()
""")
        # For all the tests, we need to remove params in lieu of generated command line
        for test in tests:
            # Generate the command line
            cli = ""
            for param in test['params']:
                # Assume long option. Possibly bad?
                cli += '--' + param + " " + test['params'][param]
            # Remove params when done
            try:
                del test['params']
            except:
                pass
            test['command_line'] = cli

        return template.render({'tests': tests, 'script_name': sys.argv[0]})



#!/usr/bin/env python
import sys
from pc import ParameterCollection
from galaxy import Galaxy
from tg import TestGenerator


class GalaxyGetOpt(object):

    def __init__(self, options=[], outputs=[], defaults={}, tests=[],
                 doc=None, argv=None):

        # Allow spec of argv here so we can override it during testing
        if argv is not None:
            self.argv = argv
        else:
            self.argv = sys.argv

        self.appid = defaults['appid']
        self.appvers = defaults['appvers']
        self.appname = defaults['appname']
        self.appdesc = defaults['appdesc']
        self.usage_desc = '%s: %s\n' % (self.appname, self.appdesc)

        parameter_collection = self.convert_opt_arr_to_params(options, outputs)

        if '--generate_galaxy_xml' in self.argv[1:]:
            ggx = Galaxy(argv=self.argv)
            print ggx.gen(param_collection=parameter_collection,
                          appinfo={'id': self.appid, 'name':
                                   self.appname, 'desc': self.appdesc,
                                   'vers': self.appvers},
                          docstring=doc, outputs=outputs,
                          options=options, defaults=defaults,
                          tests=tests)
            sys.exit(1)

        if '--gen_test' in self.argv[1:]:
            tg = TestGenerator()
            print tg.gen(tests)
            sys.exit(1)

        if '--dump_options' in self.argv[1:]:
            print self.serialize_opt_to_yaml(options=options,
                                             outputs=outputs,
                                             defaults=defaults,
                                             tests=tests)
            sys.exit(1)

        # Storing for easy access
        if '--galaxy' in self.argv[1:]:
            self.galaxy = True
        else:
            self.galaxy = False

        # Finally, store the parsed options
        self.opt = self.pC.getarg(argv_override=self.argv[1:])


    def serialize_opt_to_yaml(self, options=[], outputs=[], defaults={}, tests=[]):
        """Dumps options interface to yaml"""
        import yaml
        return yaml.dump({
            'options': options,
            'outputs': outputs,
            'defaults': defaults,
            'tests': tests,
        })

    def convert_opt_arr_to_params(self, options_array, outputs):
        """Returns a completed parameterCollection object

        Internally we add a couple other parameters that should be
        available to the user to our parameterCollection. Inside PC
        the "verbose" option is added, while here we add:

            - generate_galaxy_xml
            - galaxy
            - outfile_supporting

        The outfile_supporting variable is used as the location to
        place generated files (primary_NNNN_blah_visible_txt)

        """
        pC = ParameterCollection(self.usage_desc, vers=self.appvers)

        pC.push_params([[],['Options']])
        for option in options_array:
            pC.push_param(option)

        pC.push_param(['Output Files'])
        for output_file_option in self.fix_outputs(outputs):
            pC.push_param(output_file_option)

        pC.push_param(['generate_galaxy_xml', 'Generate Galaxy XML file',
                            {'hidden': True, 'validate': 'Flag',
                             '_show_in_galaxy': False}])
        pC.push_param(['galaxy', 'Run with galaxy-specific overrides',
                            {'hidden': True, 'validate': 'Flag',
                             '_galaxy_specific': True,
                             '_show_in_galaxy': False}])
        pC.push_param(['outfile_supporting', 'File or folder to output to',
                            {'hidden': True, 'validate': 'String', 'default':
                             '__new_file_path__', '_galaxy_specific': True,
                             '_show_in_galaxy': False}])
        self.pC = pC
        return pC

    def params(self):
        """Returns parameters, or prints help and exits if validation fails"""
        if self.pC.validate():
            return self.opt
        else:
            self.pC.argparser.print_help()
            sys.exit(2)

    def fix_outputs(self, outputs):
        """Internal function to parse specified outputs and munge them into standard options

        From a user perspective, they only specify a single output file and it
        is supposed to be handled magically. This function is a significant
        part of that magic.

        Internally, an output file is taken from being a simple object
        to four separate command line parameters, completely removed
        from the original description of the option. These four
        parameters are as follows for an example output file "of":

            -  of: the file location as specified by the user (if specified at
               all, without extension)
            -  of\_format: the format of this file. Really only applies to the top
               level file, but it's up to the dev how to handle this value
            -  of\_files\_path: the "of.files\_path" value in the cheetah template
               galaxy uses. This location can be used for storing images or other
               data for use in HTML files.
            -  of\_id: the "of.id" value in the cheetah template, used in generation
               of the complex filenames that galaxy requires to pick up on
               associated files.


        There should be no need for the user to call this method.
        """
        fixed_outputs = {}

        getopt_formatted = []
        import copy

        for output in outputs:
            name = output[0]
            desc = output[1]
            opts = output[2]

            # Primary output file
            fixed_outputs[name] = {
                'description': desc,
                'options': copy.deepcopy(opts),
            }
            fixed_outputs[name]['options']['required'] = True
            getopt_formatted.append([name, desc, opts])

            # Files path (for varCRR)
            namefp = name + "_files_path"
            namefp_desc = 'Associated HTML files for ' + name
            fixed_outputs[namefp] = {
                'description': namefp_desc,
                'options': copy.deepcopy(opts),
            }

            fixed_outputs[namefp]['options']['hidden'] = True
            fixed_outputs[namefp]['options']['value'] = name + ".files_path"
            fixed_outputs[namefp]['options']['default'] = name + ".files_path"
            fixed_outputs[namefp]['options']['required'] = True
            fixed_outputs[namefp]['options']['_galaxy_specific'] = True
            fixed_outputs[namefp]['options']['_cheetah_var'] = True
            fixed_outputs[namefp]['options']['validate'] = "String"

            getopt_formatted.append([namefp,
                                     fixed_outputs[namefp]['description'],
                                     fixed_outputs[namefp]['options']])

            # Adds the {name}_format special parameter
            namefmt = name + "_format"
            namefmt_desc = 'Associated Format for ' + name
            fixed_outputs[namefmt] = {
                'description': namefmt_desc,
                'options': copy.deepcopy(opts),
            }

            fixed_outputs[namefmt]['options']['default'] = \
                opts['default_format']
            fixed_outputs[namefmt]['options']['required'] = True
            fixed_outputs[namefmt]['options']['validate'] = "File/OutputFormat"

            getopt_formatted.append([namefmt,
                                     fixed_outputs[namefmt]['description'],
                                     fixed_outputs[namefmt]['options']])

            # Adds the {name}_id special parameter
            nameid = name + "_id"
            nameid_desc = 'Associated ID Number for ' + name
            fixed_outputs[nameid] = {
                'description': nameid_desc,
                'options': copy.deepcopy(opts),
            }

            fixed_outputs[nameid]['options']['default'] = "${%s.id}" % (name,)
            fixed_outputs[nameid]['options']['value'] = "${%s.id}" % (name,)
            fixed_outputs[nameid]['options']['required'] = True
            fixed_outputs[nameid]['options']['hidden'] = True
            fixed_outputs[nameid]['options']['validate'] = "String"
            fixed_outputs[nameid]['options']['_galaxy_specific'] = True
            fixed_outputs[namefp]['options']['_cheetah_var'] = True

            getopt_formatted.append([nameid,
                                     fixed_outputs[nameid]['description'],
                                     fixed_outputs[nameid]['options']])
        self.registered_outputs = fixed_outputs
        return getopt_formatted



from writer import Writer
from ..exceptions import WriterProcessingIncompleteError
import csv


class CSV_U(Writer):
    suffix = 'csv'

    def process(self):
        self.processed_data = self.data
        self.processing_complete = True

    def write(self):
        if self.processing_complete:
            base_name = self.OutputFilesClass.given_filename
            if base_name is None:
                base_name = ""

            for sheet in self.processed_data:
                self.OutputFilesClass.given_filename = base_name + '.' + sheet
                self.OutputFilesClass.extension = 'csv'
                next_output_file = self.OutputFilesClass.get_next_file()
                self.used_filenames.append(next_output_file)
                with open(next_output_file, 'wb') as csvfile:
                    tablewriter = csv.writer(csvfile, delimiter=',')
                    tablewriter.writerow(self.processed_data[sheet]['header'])
                    for row in self.processed_data[sheet]['data']:
                        tablewriter.writerow(row)
            self.OutputFilesClass.given_filename = base_name
        else:
            raise WriterProcessingIncompleteError("Write called but processing was not marked as complete. Not writing")



import os
from galaxygetopt.exceptions import *


class OutputFiles(object):

    def __init__(self, name="", GGO=None):

        # User supplied options
        self.name = name
        self.GGO = GGO
        if GGO is not None:
            self.galaxy = GGO.galaxy
        else:
            # Otherwise assume false
            self.galaxy = False

        # Extracted from GGO
        self.output_id = None
        self.output_label = None
        self.output_opts = {}
        # From galaxy
        self.new_file_path = None
        self.files_path = None
        self.files_id = None
        # ???
        self.parent_filename = None
        self.parent_internal_format = None
        self.parent_default_output_format = None
        # Internal use
        self.init_called = False
        # Other
        self.times_called = 0
        self.naming_strategy = 'norm'
        self.extension = None
        self.given_filename = None



        self.acceptable_formats = {
            'text/tabular': ['TSV', 'TSV_U', 'CSV', 'CSV_U', 'JSON', 'YAML',
                             'ODS','XLS', 'XLSX', 'Dumper'],
            'genomic/annotated': ['Genbank'],
            'genomic/raw': ['Fasta'],
            'text/html': ['HTML'],
            'text/plain': ['TXT', 'CONF'],
            'archive': ['tar.gz', 'zip', 'tar'],
        }

        self.format_mapping = {
            'TSV': 'tabular',
            'TSV_U': 'tabular',
            'CSV': 'tabular',
            'CSV_U': 'tabular',
            'XLS': 'data',
            'XLSX': 'data',
            'ODS': 'data',
            'JSON': 'txt',
            'YAML': 'txt',
            'Dumper': 'txt',
            'Genbank': 'txt',
            'Fasta': 'fasta',
            'HTML': 'html',
            'TXT': 'txt',
            'CONF': 'txt',
            'tar.gz': 'data',
            'zip': 'data',
            'tar': 'data',
        }

    def valid_formats(self, format_str):
        if format_str in self.acceptable_formats:
            return self.acceptable_formats[format_str]
        else:
            raise UnknownDataFormatExcpetion(format_str)

    def get_format_mapping(self, format_str):
        if format_str in self.format_mapping:
            return self.format_mapping[format_str]
        else:
            raise UnknownDataHandlerException(format_str)

    def initFromArgs(self, **kwargs):

        if self.GGO is None:
            raise UnspecifiedGGOObjectError("GGO object must be specified")

        # Grab registered outputs from GGO
        registered_outputs = self.GGO.registered_outputs
        if self.name is None:
            raise UnspecifiedOutputFileError("Author must supply a name of an output file during CRR instantiation")

        if self.name not in registered_outputs:
            raise UnknownRequestedOutputFileError("Requested output file is not known to GGO object")

        # Carrying on
        # We grab the pre-specified data regarding that output
        reg_out_params = registered_outputs[self.name]
        self.output_id = self.name
        self.output_label = reg_out_params['description']
        self.output_opts = reg_out_params['options']

        self.parent_internal_format = reg_out_params['options']['data_format']
        self.parent_default_output_format = reg_out_params['options']['default_format']
        self.parent_filename = reg_out_params['options']['default']

        # Special variables
        # --genemark "${genemark}" --genemark_format "${genemark_format}"
        # --genemark_files_path "${genemark_files_path}"
        # --genemark_id "${genemark_id}"

        # If they've specified a filename on the command line, that should
        # override the default value
        if self.GGO.opt[self.name] is not None:
            self.parent_filename = self.GGO.opt[self.name]

        # If they've specified a {str}_format option on the command line, that
        # should override the default value
        if self.GGO.opt[self.name + '_format'] is not None:
            self.parent_default_output_format = self.GGO.opt[self.name + '_format']

        # Grab supporting files path (added as new history items)
        self.new_file_path = self.GGO.opt['outfile_supporting']

        # Copy galaxy specific variables
        if self.GGO.opt[self.name + '_files_path'] is not None:
            self.files_path = self.GGO.opt[self.name + '_files_path']
        else:
            self.files_path = self.name + '_files_path'
        if self.GGO.opt[self.name + '_id'] is not None:
            self.files_id = self.GGO.opt[self.name + '_id']

        if self.GGO.opt['galaxy']:
            self.galaxy = True
        else:
            self.galaxy = False

        self.init_called = True

    def _genCRR(self, extension=None, data=None, data_format=None, format_as=None, filename=None):
        if not self.init_called:
            self.initFromArgs()

        # If the user supplied a custom extension, pull that (useful in
        # dummy/data output type)
        if extension is not None:
            self.extension = extension

        # This is a mandatory parameter
        if filename is not None:
            self.given_filename = filename
        else:
            self.given_filename = self.parent_filename

        # Allow overriding default format parameters
        df = self.parent_internal_format
        if data_format is not None:
            df = data_format

        fa= self.parent_default_output_format
        if format_as is not None:
            fa = format_as

        self.writer = self.writer_for_format(
            data_format=df, format_as=fa
        )

        # Wonder if this will translate well to py from pl
        self.writer.OutputFilesClass = self

        if data is not None:
            self.writer.data = data
        self.writer.process()
        self.writer.write()

        returned_filenames = self.writer.used_filenames
        self.bump_times_called()
        return returned_filenames

    def CRR(self, extension=None, data=None, data_format=None, format_as=None,
            filename=None):
        return self._genCRR(extension=extension, data=data,
                            data_format=data_format, format_as=format_as,
                            filename=filename)

    def subCRR(self, extension=None, data=None, data_format=None,
               format_as=None, filename=None):
        self.naming_strategy = 'sub'
        return self._genCRR(extension=extension, data=data,
                            data_format=data_format, format_as=format_as,
                            filename=filename)

    def varCRR(self, extension=None, data=None, data_format=None,
               format_as=None, filename=None):
        self.naming_strategy = 'var'
        return self._genCRR(extension=extension, data=data,
                            data_format=data_format, format_as=format_as,
                            filename=filename)

    def bump_times_called(self):
        self.times_called = self.times_called + 1
        return self.times_called

    def writer_for_format(self, data_format=None, format_as=None):
        acceptable_handlers = self.acceptable_formats[data_format]

        if format_as not in acceptable_handlers:
            raise UnacceptableOutputFormatError("""Unacceptable output format choice [%s] for
                                internal data type for type %s. Acceptable formats
                            are [%s]. Alternatively, unacceptable output
                            file."""
                            % (data_format, format_as,
                               ', '.join(acceptable_handlers)))

        if format_as == 'TXT':
            from galaxygetopt.writer.txt import TXT
            return TXT()
        elif format_as == 'TSV_U':
            from galaxygetopt.writer.tsv_u import TSV_U
            return TSV_U()
        elif format_as == 'TSV':
            from galaxygetopt.writer.tsv import TSV
            return TSV()
        elif format_as == 'CSV_U':
            from galaxygetopt.writer.csv_u import CSV_U
            return CSV_U()
        elif format_as == 'CSV':
            from galaxygetopt.writer.csv_w import CSV_W
            return CSV_W()
        elif format_as == 'Fasta':
            from galaxygetopt.writer.fasta import Fasta
            return Fasta()
        elif format_as == 'Genbank':
            from galaxygetopt.writer.genbank import Genbank
            return Genbank()
        else:
            raise NoAvailableOutputHandlerError("Could not find an appropriate output handler")

    def generate_galaxy_variable(self):
        if not os.path.isdir(self.new_file_path):
            os.makedirs(self.new_file_path)

        filename = os.path.join(
            self.new_file_path, "primary_%s_%s_visible_%s" % (self.files_id,
                                                              self.given_filename,
                                                              self.extension)
        )

        return filename

    def generate_nongalaxy_variable(self):
        filename = "%s.%s" % (self.given_filename, self.extension,)
        return filename

    def generate_galaxy_subfile(self, create_dir=True):
        """Generate galaxy subfiles, i.e., data used within an HTML report, not
        as separate history items.

        create_dir may be set to false to disable creation of directories
        (e.g., in testing)"""
        if create_dir and not os.path.isdir(self.files_path):
            os.makedirs(self.files_path)

        filename = os.path.join(
            self.files_path, "%s.%s" % (self.given_filename, self.extension)
        )

        return filename

    def generate_nongalaxy_subfile(self):
        return self.generate_galaxy_subfile()

    def get_next_file(self):
        filename = ""
        if self.galaxy:
            if self.times_called == 0:
                filename = self.parent_filename
            else:
                if self.naming_strategy == 'sub':
                    filename = self.generate_galaxy_subfile()
                elif self.naming_strategy == 'var':
                    filename = self.generate_galaxy_variable()
                else:
                    raise UnknownStrategyError("Unknown strategy for multiple output files")
        else:
            if self.times_called == 0:
                filename = "%s.%s" % (self.given_filename, self.extension)
            else:
                if self.naming_strategy == 'sub':
                    filename = self.generate_nongalaxy_subfile()
                elif self.naming_strategy == 'var':
                    filename = self.generate_nongalaxy_variable()
                else:
                    raise UnknownStrategyError("Unknown strategy for multiple output files")
        return filename



from parameter import Parameter
import xml.etree.cElementTree as ET


class Option(Parameter):

    def galaxy_input(self, xml_node):
        node_of_interest = None
        # Due to differences between python/perl, what we do here is figure out
        # which node we actually want to be talking to (main node or the repeat
        # node within it). We can then work with a generic xml_node
        possible_repeat_node = self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('select')

        if 'min' in self.child_params and self.child_params['min'] is not None:
            params['min'] = str(self.child_params['min'])
        if 'max' in self.child_params and self.child_params['max'] is not None:
            params['max'] = str(self.child_params['max'])

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

        for key in sorted(self.child_params['options']):
            subnode = ET.SubElement(node_of_interest, 'option')
            subnode.set('value', key)
            if self.default == key:
                subnode.set('selected', 'True')
            subnode.text = self.child_params['options'][key]

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, val):
        if self.required:
            if val in self.child_params['options']:
                return True
            else:
                self.errors.append("Unknown value [%s] applied to option %s" %
                                (val, self.name))
                return False
        else:
            if val is not None and val not in self.child_params['options']:
                return False
            else:
                return True



from parameter import Parameter
import xml.etree.cElementTree as ET
from galaxygetopt.outputfiles import OutputFiles


class FileOutput(Parameter):

    # # The format of the internal data structure that we're pushing to output
    # See CPT.pm for a list of these (under %acceptable)
    # has 'data_format' => ( is => 'rw', isa => 'Str' );
    # has 'default_format' => ( is => 'rw', isa => 'Str' );
    def __init__(self, name=None, short=None, multiple=False, description=None,
                 default=None, required=False, hidden=False, errors=[],
                 value=[], _galaxy_specific=False, _show_in_galaxy=True,
                 **kwargs):
        super( FileOutput, self ).__init__(name=name, short=short,
                                           multiple=multiple, description=description,
                                           default=default, required=required,
                                           hidden=hidden, errors=errors, value=value,
                                           _galaxy_specific=_galaxy_specific,
                                           _show_in_galaxy=_show_in_galaxy,
                 **kwargs)
        self._outputfile_data_access = OutputFiles()

    def data_format(self):
        if 'data_format' in self.child_params and self.child_params['data_format'] is not None:
            return self.child_params['data_format']
        else:
            return None

    def default_format(self):
        if 'default_format' in self.child_params and self.child_params['default_format'] is not None:
            return self.child_params['default_format']
        else:
            return None

    # options
    def galaxy_input(self, xml_node):
        node_of_interest = None
        # Due to differences between python/perl, what we do here is figure out
        # which node we actually want to be talking to (main node or the repeat
        # node within it). We can then work with a generic xml_node
        possible_repeat_node = self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('select')
        params['label'] = 'Format of %s' % (self.get_galaxy_cli_identifier())
        params['name'] = '%s_%s' % (self.get_galaxy_cli_identifier(), 'format')

        # Remove the value, if it is there
        try:
            del params['value']
        except KeyError:
            pass

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

        ## If we have a data format
        if self.data_format() is not None:
            # Get the list of output writers associated with that internal format
            for format_str in sorted(self._outputfile_data_access.valid_formats(self.data_format())):
                # And present them as options to the user
                option_node = ET.SubElement(node_of_interest, 'option')
                option_node.text = format_str
                option_node.set('value', format_str)
                if format_str == self.default_format():
                    option_node.set('selected', "True")

    def galaxy_output(self, xml_node):
        format_str = None
        if self.data_format() is not None:
            format_str = self.default_format()
        else:
            format_str = "data"

        node = ET.SubElement(xml_node, 'data')
        node.set('name', self.get_galaxy_cli_identifier())
        node.set('format', format_str)

        change_format = ET.SubElement(node, 'change_format')
        if self.data_format() is not None:
            for format_str in sorted(self._outputfile_data_access.valid_formats(self.data_format())):
                # And present them as options to the user
                option_node = ET.SubElement(change_format, 'when')
                option_node.set('input', '%s_%s' % (self.get_galaxy_cli_identifier(), 'format')),
                option_node.set('value', format_str)
                try:
                    option_node.set('format', self._outputfile_data_access.get_format_mapping(format_str))
                except:
                    option_node.set('format', 'data')

    def validate_individual(self, val):
        return True



from parameter import Parameter
import xml.etree.cElementTree as ET


class FileInput(Parameter):

    # has 'format'      => ( is => 'rw', isa => 'Str' );
    # has 'file_format' => ( is => 'rw', isa => 'ArrayRef' );

    # options
    def galaxy_input(self, xml_node):
        node_of_interest = None
        possible_repeat_node = self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('data')

        if 'file_format' in self.child_params and self.child_params['file_format'] is not None:
            params['format'] = ','.join(self.child_params['file_format'])

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, val):
        return True



from parameter import Parameter
import xml.etree.cElementTree as ET


class Flag(Parameter):

    # Method is overriden here ONLY because of the !required&&!not multiple
    # case,
    def galaxy_command(self):
        value = self.get_galaxy_command_identifier()
        if self.hidden and self._galaxy_specific:
            value = self.default

        # If a repeat, handle
        string = self.handle_possible_galaxy_command_repeat_start()
        # If it's required we set it to a value IF we have one. Otherwise value
        # will be the galaxy_identifier.
        if self.required:
            string += '--%s\n' % (self.get_galaxy_cli_identifier(), )
        else:
            if not self.multiple:
                string += '#if $%s:\n' % (self.get_galaxy_cli_identifier(),)

            string += '--%s\n' % (self.get_galaxy_cli_identifier(),)

            if not self.multiple:
                string += '#end if\n'
        string += self.handle_possible_galaxy_command_repeat_end()
        return string

    def galaxy_input(self, xml_node):
        node_of_interest = None
        # Due to differences between python/perl, what we do here is figure out
        # which node we actually want to be talking to (main node or the repeat
        # node within it). We can then work with a generic xml_node
        possible_repeat_node = self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('boolean')
        params['falsevalue'] = 'False'
        params['truevalue'] = 'True'
        if self.default:
            params['checked'] = 'True'
        else:
            params['checked'] = ''

        # Remove the value, if it is there
        try:
            del params['value']
        except KeyError:
            pass

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, value):
        return True



from writer import Writer
from Bio import SeqIO


class Genbank(Writer):
    suffix = 'gbk'

    def write(self):
        self.OutputFilesClass.extension = self.suffix
        next_output_file = self.OutputFilesClass.get_next_file()
        self.used_filenames.append(next_output_file)

        with open(next_output_file, 'w') as handle:
            if isinstance(self.data, list):
                for record in self.data:
                    SeqIO.write(record, handle, 'genbank')
            else:
                SeqIO.write(self.data, handle, 'genbank')




from writer import Writer
from Bio import SeqIO


class Fasta(Writer):
    suffix = 'fa'

    def write(self):
        self.OutputFilesClass.extension = self.suffix
        next_output_file = self.OutputFilesClass.get_next_file()
        self.used_filenames.append(next_output_file)
        SeqIO.write(self.data, next_output_file, 'fasta')




#!/usr/bin/env python
import xml.etree.cElementTree as ET
from ..exceptions import RepeatRequestedFromNonMultiple


class Parameter(object):

    def __init__(self, name=None, short=None, multiple=False, description=None,
                 default=None, required=False, hidden=False, errors=[],
                 value=[], _galaxy_specific=False, _show_in_galaxy=True,
                 _cheetah_var=False, **kwargs):
        self.name = name
        self.short = short
        self.multiple = multiple
        self.description = description

        # TODO: Not under test
        # Coerce non list values passed to this function into lists
        if self.multiple:
            if default is not None:
                if isinstance(default, list):
                    self.default = default
                else:
                    self.default = [default]
            else:
                self.default = []
        else:
            self.default = default
        self.required = required
        self.hidden = hidden
        self.errors = errors
        self.value = value

        self._galaxy_specific = _galaxy_specific
        self._show_in_galaxy = _show_in_galaxy
        self._cheetah_var = _cheetah_var
        # Some children will have other parameters, store them here
        self.child_params = {}
        for key in kwargs.keys():
            self.child_params[key] = kwargs[key]

    def galaxy_command(self):
        value = self.get_galaxy_command_identifier()
        # If it's specific to galaxy AND hidden from the user, then we should
        # set the value as the default. I cannot imagine a _galaxy_specific
        # !hidden situation, but we'll leave as-is in case
        if self.hidden and self._galaxy_specific:
            if self._cheetah_var:
                value = '"${%s}"' % (self.get_default(), )
            else:
                value = '"' + str(self.get_default()) + '"'
        else:
            value = '"${%s}"' % (value, )

        # If a repeat, handle
        string = self.handle_possible_galaxy_command_repeat_start()
        # If it's required we set it to a value IF we have one. Otherwise value
        # will be the galaxy_identifier.
        if self.required  or self._galaxy_specific:
            string += '--%s %s\n' % (self.get_galaxy_cli_identifier(),
                                          value)
        else:
            if not self.multiple:
                string += '#if $%s and $%s is not "None":\n' % \
                    (self.get_galaxy_cli_identifier(),
                     self.get_galaxy_cli_identifier())
            string += '--%s %s\n' % (self.get_galaxy_cli_identifier(),
                                          value)


            if not self.multiple:
                string += '#end if\n'
        string += self.handle_possible_galaxy_command_repeat_end()
        return string

    def get_galaxy_command_identifier(self):
        if self.multiple:
            return "%s.%s" % (self.get_repeat_idx_name(),
                              self.get_galaxy_cli_identifier())
        else:
            return self.get_galaxy_cli_identifier()

    def get_galaxy_cli_identifier(self):
        return self.name

    def get_value(self):
        if self.value is not None:
            if self.multiple:
                if len(self.value) > 0:
                    return self.value[0]
                else:
                    return None
            else:
                return self.value
        else:
            return None

    def get_default(self):
        # If there's a value
        if self.default is not None:
            # If it's multiple
            if self.multiple:
                # If there's more than one value, return the first
                if len(self.default) > 0:
                    return self.default[0]
                else:
                    return None
            else:
                # Otherwise just return the value
                return self.default
        else:
            return None

    def validate(self, possible_value):
        self.errors = []
        if self.multiple:
            if possible_value is not None and isinstance(possible_value, list):
                for val in possible_value:
                    self.validate_individual(val)
            elif possible_value is not None:
                self.validate_individual(possible_value)
            else:
                # If possible_value is none AND it's required, then throw an error
                if self.required:
                    self.errors.append("Must specify a value")
        else:
            self.validate_individual(possible_value)

        if len(self.errors) == 0:
            return True
        else:
            for error in self.errors:
                print "ERROR [%s] %s" % (self.get_galaxy_cli_identifier(), error)
            return False

    def get_repeat_idx_name(self):
        return 'item'

    def get_repeat_name(self):
        if self.multiple:
            return "repeat_%s" % (self.get_galaxy_cli_identifier(),)
        else:
            raise RepeatRequestedFromNonMultiple("Tried to get repeat name for non-multiple item")

    def handle_possible_galaxy_input_repeat_start(self, xml_node):
        if self.multiple:
            repeat_node = ET.SubElement(xml_node, 'repeat')
            repeat_node.set('name', self.get_repeat_name())
            # In perl, we converted this to title case?
            # $title =~ s/(\w+)/\u\L$1/g;
            repeat_node.set('title', self.get_galaxy_cli_identifier().title().replace('_',' '))
            return repeat_node
        else:
            return None

    def handle_possible_galaxy_command_repeat_start(self):
        if self.multiple:
            return "#for $%s in $%s:\n" % (self.get_repeat_idx_name(),
                                           self.get_repeat_name())
        else:
            return ''

    def handle_possible_galaxy_command_repeat_end(self):
        if self.multiple:
            return "#end for\n"
        else:
            return ''

    def get_default_input_parameters(self, paramType):
        params = {
            'name': self.get_galaxy_cli_identifier(),
            'optional': self.is_optional_galaxy(),
            'label': self.get_galaxy_cli_identifier(),
            'help': self.get_description(),
            'type': paramType,
        }

        if self.get_default() is not None:
            params['value'] = self.get_default()

            # Make sure it's not None as ET cannot serialize NoneTypes
            if params['value'] is None:
                params['value'] = ""

            if type(params['value']) is not str:
                params['value'] = str(params['value'])

        return params

    def get_description(self):
        if self.description is not None:
            return self.description
        else:
            return "Author failed to supply a description"

    def get_all_keys(self):
        kd = {}
        if self.name is not None:
            kd['name'] = self.name
        if self.short is not None:
            kd['short'] = self.short
        if self.multiple is not None:
            kd['multiple'] = self.multiple
        if self.description is not None:
            kd['description'] = self.description
        if self.default is not None:
            kd['default'] = self.default
        if self.required is not None:
            kd['required'] = self.required
        if self.hidden is not None:
            kd['hidden'] = self.hidden
        if self.errors is not None:
            kd['errors'] = self.errors
        if self.value is not None:
            kd['value'] = self.value
        for p in self.child_params:
            kd[p] = self.child_params[p]
        return kd

    # "requires"
    def is_optional_galaxy(self):
        if self.required:
            return "False"
        else:
            return "True"

    # These all are implemented by children
    def validate_individual(self, value):
        return True

    def galaxy_input(self):
        return None

    def galaxy_output(self):
        return None



from parameter import Parameter
import xml.etree.cElementTree as ET


class String(Parameter):
    def galaxy_input(self, xml_node):
        node_of_interest = None
        possible_repeat_node = self.handle_possible_galaxy_input_repeat_start(xml_node)
        if possible_repeat_node is not None:
            node_of_interest = ET.SubElement(possible_repeat_node, 'param')
        else:
            node_of_interest = ET.SubElement(xml_node, 'param')

        params = self.get_default_input_parameters('text')

        # Copy params over
        for key in params:
            node_of_interest.set(key, params[key])

    def galaxy_output(self, xml_node):
        return None

    def validate_individual(self, value):
        return True



from option import Option


class GenomicTagOption(Option):

    def __init__(self, *args, **kwargs):
        self.valid_keys = ["-10_signal", "-35_signal", "3'UTR", "5'UTR",
                           "CAAT_signal", "CDS", "C_region", "D-loop",
                           "D_segment", "GC_signal", "J_segment", "LTR",
                           "N_region", "RBS", "STS", "S_region", "TATA_signal",
                           "V_region", "V_segment", "assembly_gap",
                           "attenuator", "enhancer", "exon", "gap", "gene",
                           "iDNA", "intron", "mRNA", "mat_peptide", "misc_RNA",
                           "misc_binding", "misc_difference", "misc_feature",
                           "misc_recomb", "misc_signal", "misc_structure",
                           "mobile_element", "modified_base", "ncRNA",
                           "old_sequence", "operon", "oriT", "polyA_signal",
                           "polyA_site", "precursor_RNA", "prim_transcript",
                           "primer_bind", "promoter", "protein_bind", "rRNA",
                           "rep_origin", "repeat_region", "sig_peptide",
                           "source", "stem_loop", "tRNA", "terminator",
                           "tmRNA", "transit_peptide", "unsure", "variation",
                           # These are "custom"
                           "whole", "all"]
        self.validKeySet = {x: x for x in sorted(self.valid_keys)}
        kwargs['options'] = self.validKeySet
        Option.__init__(self, *args, **kwargs)
